
/* jlh */

/* this file is GPL'ed, see README */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <string.h>

#include "wmlessgeneral.h"

/* max pixel value in buffers */
#define CEIL 4095

#define N_BLOBS 5

typedef struct
{
	int r, g, b;
} COLOR;

static COLOR the_colors[16] =
{
	{ 0, 0, 0 }, { 0, 0, 127 }, { 0, 127, 0 }, { 0, 127, 127 },
	{ 127, 0, 0 }, { 127, 0, 127 }, { 127, 127, 0 }, { 191, 191, 191 },
	{ 127, 127, 127 }, { 0, 0, 255 }, { 0, 255, 0 }, { 0, 255, 255 },
	{ 255, 0, 0 }, { 255, 0, 255 }, { 255, 255, 0 }, { 255, 255, 255 }
};

static short buf[4096];  /* rendering buffer */
static short def[4096];  /* backgound (light borders & edges) */
static short blob[1024]; /* a blob */
static short sin_table[1024];
static short blob_vx[N_BLOBS], blob_vy[N_BLOBS]; /* blob speed */
static int clock;

/* parameters for get_value() */
static int alpha, beta, offset, radius2, maximum, hardness;

static unsigned long my_color_map[32]; /* index for our colors */
static char current_color[3];          /* current colors.  each from 0 to 15 */

static XImage *ximage;


/* set parameters for get_value() */

static void set_params(int r2, int h, int m)
{
	int t;

	radius2 = r2;
	maximum = m;
	hardness = h;
	offset = maximum / hardness;
	t = maximum - offset * 2;
	alpha = (radius2 * (maximum + offset) - 256 * offset + t / 2) / t;
	beta = offset * (256 - alpha);
}


/* funtion doing the smooth falloff.  looks something like that:
 *
 *     ^
 *     |
 *     |
 * max-|-----
 *     |     \_
 *     |       \__
 *     |          \___
 *     +------------------>
 *          ^
 *          `radius
 */

static int get_value(int r2)
{
	int c;

	c = (r2 < radius2) ? maximum : beta / (r2 - alpha) - offset;

	return(c <= 0 ? 0 : c);
	/* clamping to CEIL not needed */
}



static void build_blob()
{
	int x, y;
	short *blob_ptr;

	set_params(2, 16, CEIL);
	blob_ptr = blob;

	for (y = -16; y < 16; y++)
	{
		for (x = -16; x < 16; x++)
		{
			*blob_ptr++ = get_value(x * x + y * y);
		}
	}
}



static void build_sin_table()
{
	short i;

	for (i = 0; i < 1024; i++)
	{
		sin_table[i] = (short)(sin((float)i *
				M_PI * 2.0 / 1024.0) * 1024.0 + 0.5);
	}
}


/* look up value in sine table */

static int sin_t(int a)
{
	return((int)sin_table[a & 0x3ff]);
}



static void build_background()
{
	short i, j;
	int c;

	memset(def, 0, 4096);
	set_params(0, 32, CEIL/2);

	for (i = 0; i < 16; i++)
	{
		c = get_value(i * i);

		for (j = 0; j < 64; j++)
		{
			/* no clamping, hope it's ok */
			def[j * 64 + i] += c;
			def[j * 64 + 63 - i] += c;
			def[i * 64 + j] += c;
			def[(63 - i) * 64 + j] += c;
		}
	}
}


/* set the speed of each blob */

static void init_speed()
{
	int i;

	for (i = 0; i < N_BLOBS; i++)
	{
		blob_vx[i] = rand() % 20 + 1;
		blob_vy[i] = rand() % 20 + 1;
	}
}


/* put a blob in buf at the given location */

static void put_blob(int px, int py)
{
	short *blob_ptr, *buf_ptr;
	int sx, sy, x;
	int blob_jump, buf_jump;
	int c;

	if (px <= -16 || py <= -16 || px >= 80 || py >= 80)
	{
		return;
	}

	blob_ptr = blob;
	buf_ptr = buf + px - 16 + (py - 16) * 64;

	if (px < 16)
	{
		sx = 16 + px;
		blob_ptr += 16 - px;
		buf_ptr += 16 - px;
	}
	else if (px > 48)
	{
		sx = 80 - px;
	}
	else
	{
		sx = 32;
	}

	blob_jump = 32 - sx;
	buf_jump = 64 - sx;

	if (py < 16)
	{
		sy = 16 + py;
		blob_ptr += (16 - py) * 32;
		buf_ptr += (16 - py) * 64;
	}
	else if (py > 48)
	{
		sy = 80 - py;
	}
	else
	{
		sy = 32;
	}

	while (sy--)
	{
		x = sx;

		while (x--)
		{
			c = *buf_ptr + *blob_ptr +
				*buf_ptr * *blob_ptr * 16 / CEIL;
			*buf_ptr = (c < CEIL) ? c : CEIL;
			buf_ptr++;
			blob_ptr++;
		}

		buf_ptr += buf_jump;
		blob_ptr += blob_jump;
	}
}


/* draw all blobs in buf */

static void draw()
{
	int x, y, i;

	/* copy background in buf */
	memcpy(buf, def, 4096 * sizeof(short));

	for (i = 0; i < N_BLOBS; i++)
	{
		x = sin_t(clock * blob_vx[i]) / 32 + 32;
		y = sin_t(clock * blob_vy[i] + 64) / 32 + 32;
		put_blob(x, y);
	}

	clock++;
}


/* copy content of buf to screen */

static void show()
{
	int x, y;
	short *p;

	p = buf;

	for (y = 0; y < 64; y++)
	{
		for (x = 0; x < 64; x++)
		{
			XPutPixel(ximage, x, y, my_color_map[*p++ * 31 / CEIL]);
		}
	}

        XPutImage(display, iconwin, NormalGC, ximage, 0, 0, 0, 0, 64, 64);
	XFlush(display);
}


/* interpolate color.  i from 0 to max, color from c1 to c2 */

static void interpolate(int i, int max, COLOR *c1, COLOR *c2, COLOR *color)
{
	color->r = (c1->r * (max - i) + c2->r * i + max / 2) / max;
	color->g = (c1->g * (max - i) + c2->g * i + max / 2) / max;
	color->b = (c1->b * (max - i) + c2->b * i + max / 2) / max;
}


/* generate index map */

static void alloc_colors()
{
	XColor xcolor;
	COLOR color;
	COLOR c[3];
	int i;

	c[0] = the_colors[current_color[0]];
	c[1] = the_colors[current_color[1]];
	c[2] = the_colors[current_color[2]];

        for (i = 0; i < 32; i++)
	{
		if (i <= 16)
		{
			interpolate(i, 16, c, c + 1, &color);
		}
		else
		{
			interpolate(i - 16, 15, c + 1, c + 2, &color);
		}

		xcolor.red = color.r << 8;
		xcolor.green = color.g << 8;
		xcolor.blue = color.b << 8;
		XAllocColor(display, color_map, &xcolor);
		my_color_map[i] = xcolor.pixel;
	}
}


/* handle pending events */

static void do_pending()
{
	XEvent event;
	char *c;

	while (XPending(display))
	{
		XNextEvent(display, &event);

		switch (event.type)
		{
			case DestroyNotify:
				XCloseDisplay(display);
				exit(0);
				break;

			case ButtonPress:
				c = current_color + 2;

				switch (event.xbutton.button)
				{
					case Button2:
						c = current_color + 1;
						break;

					case Button3:
						c = current_color;
						break;
				}

				(*c)++, *c %= 16;
				alloc_colors();
		}
	}
}


/* main loop, endless loop */

void do_demo(void)
{
	while (1)
	{
		draw();
		show();
		do_pending();
		usleep(50000);
	}
}


/* scan args */

void scan_args(int argc,char **argv)
{
	int i;

	for (i = 0; i < argc; i++)
	{
		char *arg = argv[i];

		if (*arg == '-')
		{
			switch (arg[1])
			{
				case 'v':
					/* TODO: write some credits */
					exit(0);
					break;

				default:
					;
					break;
			}
		}
	}
}


/* home sweet home */

int main(int argc, char *argv[])
{
	char mask[512];
	int i;

	/* init */

	srand(getpid());
	scan_args(argc, argv);
	build_sin_table();
	build_blob();
	build_background();
	init_speed();

	/* build mask for window */

	memset(mask, 0x00, 24);
	memset(mask + 24, 0xff, 466);
	memset(mask + 488, 0x00, 24);

	for (i = 3; i < 61; i++)
	{
		mask[i * 8] = 0xf8;
		mask[i * 8 + 7] = 0x1f;
	}

	/* init window */
	openXwindow(argc, argv, mask, 64, 64);
	ximage = XCreateImage(display, visual, depth,
			ZPixmap, 0, NULL, 64, 64, 32, 0);
	ximage->data = (char *)malloc(ximage->bytes_per_line * 64);

	/* init colors */
	current_color[0] = 0;
	current_color[1] = 9;
	current_color[2] = 15;
	alloc_colors();

	/* run it */
	clock = 0;
	do_demo();

	return(0);
}

