
/* jlh */

/* this file is GPL'ed, see README */

/* this file originally was taken from wmgeneral.c
   by Martijn Pieterse (pieterse@xs4all.nl) */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "wmlessgeneral.h"

Display *display;
Visual *visual;
int depth;
int color_map;
Window root, iconwin, win;

static int screen;
static XSizeHints mysizehints;
static XWMHints mywmhints;
static Pixel back_pix, fore_pix;
static char *Geometry = "";
GC NormalGC;
static Pixmap pixmask;

void openXwindow(int argc, char **argv, char *pixmask_bits,
		int pixmask_width, int pixmask_height)
{
	XClassHint class_hint;
	char *display_name;
	XTextProperty name;
	XGCValues gcv;
	char *geometry = NULL;
	int dummy=0;
	int i;
	int white, black;

	display_name = NULL;

	for (i = 1; argv[i]; i++)
	{
		if (!strcmp(argv[i], "-display"))
		{
			display_name = argv[++i];
		}
		else if (!strcmp(argv[i], "-geometry"))
		{
			geometry = argv[++i];
		}
	}

	display = XOpenDisplay(display_name);

	if (!display)
	{
		fprintf(stderr, "%s: can't open display %s\n", *argv,
				XDisplayName(display_name));
		exit(1);
	}

	screen = DefaultScreen(display);
	root = RootWindow(display, screen);
	visual = DefaultVisual(display, screen);
	depth = DefaultDepth(display, screen);
	color_map = DefaultColormap(display, 0);
	black = BlackPixel(display, screen);
	white = WhitePixel(display, screen);

	/* Create a window to hold the stuff */
	mysizehints.flags = USSize | USPosition;
	mysizehints.x = 0;
	mysizehints.y = 0;

	XWMGeometry(display, screen, Geometry, NULL, 1, &mysizehints,
			&mysizehints.x, &mysizehints.y, &mysizehints.width,
			&mysizehints.height, &dummy);

	mysizehints.width = 64;
	mysizehints.height = 64;

	win = XCreateSimpleWindow(display, root, mysizehints.x, mysizehints.y,
			mysizehints.width, mysizehints.height, 1, white, black);

	iconwin = XCreateSimpleWindow(display, win, mysizehints.x,
			mysizehints.y, mysizehints.width, mysizehints.height,
			1, white, black);

	XSetWMNormalHints(display, win, &mysizehints);
	class_hint.res_name = *argv;
	class_hint.res_class = *argv;
	XSetClassHint(display, win, &class_hint);

	XSelectInput(display, win, StructureNotifyMask);
	XSelectInput(display, iconwin, StructureNotifyMask | ButtonPressMask);

	if (!XStringListToTextProperty(argv, 1, &name))
	{
		fprintf(stderr, "%s: can't allocate window name\n", *argv);
		exit(1);
	}

	XSetWMName(display, win, &name);

	/* Create GC for drawing */

	gcv.foreground = fore_pix;
	gcv.background = back_pix;
	gcv.graphics_exposures = 0;
	NormalGC = XCreateGC(display, root, GCForeground | GCBackground |
			GCGraphicsExposures, &gcv);

	pixmask = XCreateBitmapFromData(display, win, pixmask_bits,
			pixmask_width, pixmask_height);
	XShapeCombineMask(display, iconwin, ShapeBounding, 0, 0,
			pixmask, ShapeSet);

	mywmhints.initial_state = WithdrawnState;
	mywmhints.icon_window = iconwin;
	mywmhints.icon_x = mysizehints.x;
	mywmhints.icon_y = mysizehints.y;
	mywmhints.window_group = win;
	mywmhints.flags = StateHint | IconWindowHint | IconPositionHint
		| WindowGroupHint;
	XSetWMHints(display, win, &mywmhints);

	XSetCommand(display, win, argv, argc);
	XMapWindow(display, win);
}

